#include "cAHDSRComponent.h"

cAHDSRComponent::cAHDSRComponent()
{
    initAnchors( eAnchorCount );

	// start Anchor is locked at the bottom, can move in the horizontal only
    mEnvelopeAnchors[ eStartAnchor ]->mX.mLocked = false;
    mEnvelopeAnchors[ eStartAnchor ]->mY.mLocked = true;
    mEnvelopeAnchors[ eStartAnchor ]->mX.mPercentage  = 0.0;
    mEnvelopeAnchors[ eStartAnchor ]->mY.mPercentage = 0.0;

	// attack Anchor is locked at the top, can move in the horizontal only
    mEnvelopeAnchors[ eAttackAnchor ]->mX.mLocked = false;
    mEnvelopeAnchors[ eAttackAnchor ]->mY.mLocked = true;
    mEnvelopeAnchors[ eAttackAnchor ]->mX.mPercentage  = 0.0;
    mEnvelopeAnchors[ eAttackAnchor ]->mY.mPercentage = 1.0;
    mEnvelopeAnchors[ eAttackAnchor ]->mY.mFollowRight = true;

	// hold Anchor is locked at the tope can move in the horizontal only
    mEnvelopeAnchors[ eHoldAnchor ]->mX.mLocked = false;
    mEnvelopeAnchors[ eHoldAnchor ]->mY.mLocked = true;
    mEnvelopeAnchors[ eHoldAnchor ]->mX.mPercentage  = 0.33;
    mEnvelopeAnchors[ eHoldAnchor ]->mY.mPercentage = 1.0;
    mEnvelopeAnchors[ eHoldAnchor ]->mY.mFollowLeft = true;

	// decay Anchor can move in both horizontal and vertical
    mEnvelopeAnchors[ eDecayAnchor ]->mX.mLocked = false;
    mEnvelopeAnchors[ eDecayAnchor ]->mY.mLocked = false;
    mEnvelopeAnchors[ eDecayAnchor ]->mX.mPercentage  = 0.66;
    mEnvelopeAnchors[ eDecayAnchor ]->mY.mPercentage = 1.0;
    mEnvelopeAnchors[ eDecayAnchor ]->mY.mFollowRight = true;

	// sustain Anchor can move both horizontal and vertical
    mEnvelopeAnchors[ eSustainAnchor ]->mX.mLocked = false;
    mEnvelopeAnchors[ eSustainAnchor ]->mY.mLocked = false;
    mEnvelopeAnchors[ eSustainAnchor ]->mX.mPercentage  = 1.0;
    mEnvelopeAnchors[ eSustainAnchor ]->mY.mPercentage = 1.0;
    mEnvelopeAnchors[ eSustainAnchor ]->mY.mFollowLeft = true;

	// release Anchor is locked at the bottom, can move the horizontal only
    mEnvelopeAnchors[ eReleaseAnchor ]->mX.mLocked = false;
    mEnvelopeAnchors[ eReleaseAnchor ]->mY.mLocked = true;
    mEnvelopeAnchors[ eReleaseAnchor ]->mX.mPercentage  = 1.0;
    mEnvelopeAnchors[ eReleaseAnchor ]->mY.mPercentage = 0.0;

    recalculateCoordinates();
}

cAHDSRComponent::~cAHDSRComponent()
{
    // doing nothing yet
}

void cAHDSRComponent::setTimePercents( double startTimePercent, double attackTimePercent, double holdTimePercent, double decayTimePercent, double sustainTimePercent, double releaseTimePercent )
{
    setAnchorXPercent( eStartAnchor,  startTimePercent );
    setAnchorXPercent( eAttackAnchor, attackTimePercent );
    setAnchorXPercent( eHoldAnchor, holdTimePercent );
    setAnchorXPercent( eDecayAnchor, decayTimePercent );
    setAnchorXPercent( eSustainAnchor, sustainTimePercent );
    setAnchorXPercent( eReleaseAnchor, releaseTimePercent );

    repaint();
}

void cAHDSRComponent::setStartTimePercent( double startTimePercent )
{
    setAnchorXPercent( eStartAnchor, startTimePercent );

    repaint();
}

void cAHDSRComponent::setAttackTimePercent( double attackTimePercent )
{
    setAnchorXPercent( eAttackAnchor, attackTimePercent );

    repaint();
}

void cAHDSRComponent::setHoldTimePercent( double holdTimePercent )
{
    setAnchorXPercent( eHoldAnchor, holdTimePercent );

    repaint();
}

void cAHDSRComponent::setDecayTimePercent( double decayTimePercent )
{
    setAnchorXPercent( eDecayAnchor, decayTimePercent );

    repaint();
}

void cAHDSRComponent::setSustainTimePercent( double sustainTimePercent )
{
    setAnchorXPercent( eSustainAnchor, sustainTimePercent );

    repaint();
}

void cAHDSRComponent::setSustainLevelPercent( double sustainLevel )
{
    setAnchorYPercent( eDecayAnchor, sustainLevel );
    setAnchorYPercent( eSustainAnchor, sustainLevel );

    repaint();
}

void cAHDSRComponent::setReleaseTimePercent( double releaseTimePercent )
{
    setAnchorXPercent( eReleaseAnchor, releaseTimePercent );

    repaint();
}
