#ifndef _C_ENVELOPE_COMPONENT_
#define _C_ENVELOPE_COMPONENT_

#include "../JuceLibraryCode/JuceHeader.h"

class cAxis
{
public:
    cAxis(void)
        : mPercentage(0.0),
          mLocked(false),
          mFollowLeft(false),
          mFollowRight(false)
    {
    }

    double mPercentage;
    bool   mLocked;
    bool   mFollowLeft;
    bool   mFollowRight;

private:
};

class cEnvelopeAnchor
{
public:
    cEnvelopeAnchor(void);

    void Display( Graphics& g, int xOffset, int yOffset );
    void UpdateDisplayState( void );
    void drawCircle(Graphics& g, float centerX, float centerY, float startSize, float size, Colour color);

    bool IsActive(void) { return mActive; }
    void SetActive(bool active) { 
        mActive = active;
        UpdateDisplayState();
    }

    bool IsSelected(void) { return mSelected; }
    void SetSelected(bool selected) { mSelected = selected; UpdateDisplayState(); }

    bool IsEnabled(void) { return mEnabled; }
    void SetEnabled(bool enabled) { mEnabled = enabled; UpdateDisplayState(); }

    int DistanceMovedX(int xMove);
    int DistanceMovedY(int yMove);

    void SetMaxX(int maxX) { mMaxX = maxX; }
    void SetMaxY(int maxY) { mMaxY = maxY; }

    void SetSize(int size) { mSize = size; }
    int  GetSize(void) { return mSize; }

    void recalculateX(int offset, int displayWidth)
    {
        mCurX = offset + (long)(displayWidth * mX.mPercentage);
    }
    void recalculateY(int offset, int displayHeight)
    {
        mCurY = offset + (displayHeight - (long)(displayHeight * mY.mPercentage));
    }
    void recalculateXPercent(int offset, int displayWidth)
    {
        mX.mPercentage = ((double)mCurX - (double)offset) / (double)displayWidth;
    }
    void recalculateYPercent(int offset, int displayHeight)
    {
        mY.mPercentage = ((double)displayHeight - ((double)mCurY - (double)offset)) / (double)displayHeight;
    }

    //////////////////////////////////
    cAxis mX;
    cAxis mY;

    int mCurX;
    int mCurY;

    int mMaxX;
    int mMaxY;

    cEnvelopeAnchor* mAnchorToLeft;
    cEnvelopeAnchor* mAnchorToRight;

    Colour mColor;

private:
    int mSize;
    bool mEnabled;
    bool mActive;
    bool mSelected;
};

class cEnvelopeComponent : public Component,
                           public ChangeBroadcaster
{
public:
    cEnvelopeComponent();
    ~cEnvelopeComponent();

    void initAnchors( long numberOfEnvelopeAnchors );
    void setAnchorXPercent( long anchorIndex, double xPercent );
    void setAnchorYPercent( long anchorIndex, double yPercent );

    double getAnchorXPercent( long anchorIndex );
    double getAnchorYPercent( long anchorIndex );

    void addAnchor(cEnvelopeAnchor* anchor, int index = -1);
    cEnvelopeAnchor* deleteAnchor(int index);
    virtual void rightMouseClickOnAnchor( const MouseEvent &e );
    virtual void rightMouseClickOnBackground( const MouseEvent &e );

    virtual void paintBackground( Graphics& g );

    // overrides from base classes
    void paint( Graphics& g );
    void resized();

    // mouse overrides
    void mouseMove( const MouseEvent &e );
    void mouseDown( const MouseEvent &e );
    void mouseUp( const MouseEvent &e );
    void mouseDrag( const MouseEvent &e );
    void mouseDownOnAnchor(const MouseEvent &e);
    void mouseDownOnBackground(const MouseEvent &e);
    void mouseUpDraggingAnchor(const MouseEvent &e);
    void mouseUpRectangleSelection(const MouseEvent &e);
    void mouseDragAnchor(const MouseEvent &e);
    void mouseDragRectangleSelection(const MouseEvent &e);

    void recalculateCoordinates( void );
    //==============================================================================
    juce_UseDebuggingNewOperator

protected:
    OwnedArray<cEnvelopeAnchor> mEnvelopeAnchors;

    long mEnvelopeDisplayAreaWidth;
    long mEnvelopeDisplayAreaHeight;
    long mBorderWidth;
    long mAnchorSize;
    long mOffset;
    int mStartX;
    int mStartY;
    int mStartRectSelectX;
    int mStartRectSelectY;
    int mCurRectSelectX;
    int mCurRectSelectY;

private:
    #define kDefaultBorderWidth 1
    #define kDefaultHandleSize 9

    #define kDefaultBackgroundColour     Colours::lightgrey
    #define kDefaultBorderColour         Colours::black
    #define kDefaultLineColour           Colours::cyan
    #define kDefaultGlowColour           Colours::cyan
    #define kDefaultNormalHandleColour   Colours::royalblue
    #define kDefaultActiveHandleColour   Colours::red 
    #define kDefaultDisabledHandleColour Colours::dimgrey

    #define kNoAnchor NULL

    // data
    cEnvelopeAnchor* mCurActiveAnchor;
    cEnvelopeAnchor* mDraggingAnchor;

    Colour  mBackgroundColour;
    Colour  mBorderColour;
    Colour  mLineColour;
    Colour  mGlowColour;
    Colour  mNormalHandleColour;
    Colour  mActiveHandleColour;
    Colour  mDisabledHandleColour;

    Image   mBackgroundImage;

    bool mResetWhenDone;
};

#endif // _C_ENVELOPE_COMPONENT_
